@@ -2,7 +2,7 @@
    system math.h classification macros.
 
    Copyright (C) 2006 IBM Corporation.
-   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2009-2014 Free Software Foundation, Inc.
 
    This file is part of the Decimal Floating Point C Library.
 
@@ -179,6 +179,12 @@ extern int __isinfd32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
 extern int isfinited32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
 extern int __isfinited32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
 
+extern int __issignalingd32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
+extern int issignalingd32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
+
+extern int fpclassifyd32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
+extern int __fpclassifyd32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
+
 /* Deprecated but older uses of math.h may have invocations of these if they
  * used the polymorphic finite().  */
 extern int finited32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
@@ -213,8 +219,6 @@ extern int islessgreaterd32(_Decimal32 x, _Decimal32 y) __THROW __attribute__ ((
 
 extern int isunorderedd32 (_Decimal32 x, _Decimal32 y) __THROW __attribute__ ((__const__));
 
-extern int __fpclassifyd32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
-
 extern int __signbitd32 (_Decimal32 __value) __THROW __attribute__ ((__const__));
 
 extern _Decimal32 j0d32 (_Decimal32) __THROW;
@@ -271,6 +275,12 @@ extern _Decimal32 __scalbnd32 (_Decimal32 __x, int __n) __THROW;
 extern int ilogbd32 (_Decimal32 __x) __THROW;
 extern int __ilogbd32 (_Decimal32 __x) __THROW;
 
+extern long int llogbd32 (_Decimal32 __x) __THROW;
+extern long int __llogbd32 (_Decimal32 __x) __THROW;
+
+extern long long int llquantexpd32 (_Decimal32 x) __THROW;
+extern long long int __llquantexpd32 (_Decimal32 x) __THROW;
+
 extern _Decimal32 scalblnd32 (_Decimal32 __x, long int __n) __THROW;
 extern _Decimal32 __scalblnd32 (_Decimal32 __x, long int __n) __THROW;
 
@@ -280,6 +290,9 @@ extern _Decimal32 __nearbyintd32 (_Decimal32 __x) __THROW;
 extern _Decimal32 roundd32 (_Decimal32 __x) __THROW __attribute__ ((__const__));
 extern _Decimal32 __roundd32 (_Decimal32 __x) __THROW __attribute__ ((__const__));
 
+extern _Decimal32 roundevend32 (_Decimal32 __x) __THROW __attribute__ ((__const__));
+extern _Decimal32 __roundevend32 (_Decimal32 __x) __THROW __attribute__ ((__const__));
+
 extern _Decimal32 truncd32 (_Decimal32 __x) __THROW __attribute__ ((__const__));
 extern _Decimal32 __truncd32 (_Decimal32 __x) __THROW __attribute__ ((__const__));
 
@@ -316,6 +329,9 @@ extern _Decimal32 __scalbd32 (_Decimal32 __x, _Decimal32 __n) __THROW;
 extern _Decimal32 quantized32 (_Decimal32 __x, _Decimal32 __y) __THROW;
 extern _Decimal32 __quantized32 (_Decimal32 __x, _Decimal32 __y) __THROW;
 
+extern _Decimal32 quantumd32 (_Decimal32 __x) __THROW;
+extern _Decimal32 __quantumd32 (_Decimal32 __x) __THROW;
+
 extern _Bool samequantumd32 (_Decimal32 __x, _Decimal32 __y) __THROW;
 extern _Bool __samequantumd32 (_Decimal32 __x, _Decimal32 __y) __THROW;
 
@@ -421,6 +437,12 @@ extern int __isinfd64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
 extern int isfinited64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
 extern int __isfinited64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
 
+extern int __issignalingd64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
+extern int issignalingd64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
+
+extern int fpclassifyd64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
+extern int __fpclassifyd64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
+
 /* Deprecated but older uses of math.h may have invocations of these if they
  * used the polymorphic finite().  */
 extern int finited64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
@@ -455,8 +477,6 @@ extern int islessgreaterd64(_Decimal64 x, _Decimal64 y) __THROW __attribute__ ((
 
 extern int isunorderedd64 (_Decimal64 x, _Decimal64 y) __THROW __attribute__ ((__const__));
 
-extern int __fpclassifyd64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
-
 extern int __signbitd64 (_Decimal64 __value) __THROW __attribute__ ((__const__));
 
 extern _Decimal64 j0d64 (_Decimal64) __THROW;
@@ -513,6 +533,12 @@ extern _Decimal64 __scalbnd64 (_Decimal64 __x, int __n) __THROW;
 extern int ilogbd64 (_Decimal64 __x) __THROW;
 extern int __ilogbd64 (_Decimal64 __x) __THROW;
 
+extern long int llogbd64 (_Decimal64 __x) __THROW;
+extern long int __llogbd64 (_Decimal64 __x) __THROW;
+
+extern long long int llquantexpd64 (_Decimal64 x) __THROW;
+extern long long int __llquantexpd64 (_Decimal64 x) __THROW;
+
 extern _Decimal64 scalblnd64 (_Decimal64 __x, long int __n) __THROW;
 extern _Decimal64 __scalblnd64 (_Decimal64 __x, long int __n) __THROW;
 
@@ -522,6 +548,9 @@ extern _Decimal64 __nearbyintd64 (_Decimal64 __x) __THROW;
 extern _Decimal64 roundd64 (_Decimal64 __x) __THROW __attribute__ ((__const__));
 extern _Decimal64 __roundd64 (_Decimal64 __x) __THROW __attribute__ ((__const__));
 
+extern _Decimal64 roundevend64 (_Decimal64 __x) __THROW __attribute__ ((__const__));
+extern _Decimal64 __roundevend64 (_Decimal64 __x) __THROW __attribute__ ((__const__));
+
 extern _Decimal64 truncd64 (_Decimal64 __x) __THROW __attribute__ ((__const__));
 extern _Decimal64 __truncd64 (_Decimal64 __x) __THROW __attribute__ ((__const__));
 
@@ -558,6 +587,9 @@ extern _Decimal64 __scalbd64 (_Decimal64 __x, _Decimal64 __n) __THROW;
 extern _Decimal64 quantized64 (_Decimal64 __x, _Decimal64 __y) __THROW;
 extern _Decimal64 __quantized64 (_Decimal64 __x, _Decimal64 __y) __THROW;
 
+extern _Decimal64 quantumd64 (_Decimal64 __x) __THROW;
+extern _Decimal64 __quantumd64 (_Decimal64 __x) __THROW;
+
 extern _Bool samequantumd64 (_Decimal64 __x, _Decimal64 __y) __THROW;
 extern _Bool __samequantumd64 (_Decimal64 __x, _Decimal64 __y) __THROW;
 
@@ -663,6 +695,12 @@ extern int __isinfd128 (_Decimal128 __value) __THROW __attribute__ ((__const__))
 extern int isfinited128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
 extern int __isfinited128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
 
+extern int __issignalingd128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
+extern int issignalingd128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
+
+extern int fpclassifyd128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
+extern int __fpclassifyd128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
+
 /* Deprecated but older uses of math.h may have invocations of these if they
  * used the polymorphic finite().  */
 extern int finited128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
@@ -697,8 +735,6 @@ extern int islessgreaterd128(_Decimal128 x, _Decimal128 y) __THROW __attribute__
 
 extern int isunorderedd128 (_Decimal128 x, _Decimal128 y) __THROW __attribute__ ((__const__));
 
-extern int __fpclassifyd128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
-
 extern int __signbitd128 (_Decimal128 __value) __THROW __attribute__ ((__const__));
 
 extern _Decimal128 j0d128 (_Decimal128) __THROW;
@@ -755,6 +791,12 @@ extern _Decimal128 __scalbnd128 (_Decimal128 __x, int __n) __THROW;
 extern int ilogbd128 (_Decimal128 __x) __THROW;
 extern int __ilogbd128 (_Decimal128 __x) __THROW;
 
+extern long int llogbd128 (_Decimal128 __x) __THROW;
+extern long int __llogbd128 (_Decimal128 __x) __THROW;
+
+extern long long int llquantexpd128 (_Decimal128 x) __THROW;
+extern long long int __llquantexpd128 (_Decimal128 x) __THROW;
+
 extern _Decimal128 scalblnd128 (_Decimal128 __x, long int __n) __THROW;
 extern _Decimal128 __scalblnd128 (_Decimal128 __x, long int __n) __THROW;
 
@@ -764,6 +806,9 @@ extern _Decimal128 __nearbyintd128 (_Decimal128 __x) __THROW;
 extern _Decimal128 roundd128 (_Decimal128 __x) __THROW __attribute__ ((__const__));
 extern _Decimal128 __roundd128 (_Decimal128 __x) __THROW __attribute__ ((__const__));
 
+extern _Decimal128 roundevend128 (_Decimal128 __x) __THROW __attribute__ ((__const__));
+extern _Decimal128 __roundevend128 (_Decimal128 __x) __THROW __attribute__ ((__const__));
+
 extern _Decimal128 truncd128 (_Decimal128 __x) __THROW __attribute__ ((__const__));
 extern _Decimal128 __truncd128 (_Decimal128 __x) __THROW __attribute__ ((__const__));
 
@@ -800,6 +845,9 @@ extern _Decimal128 __scalbd128 (_Decimal128 __x, _Decimal128 __n) __THROW;
 extern _Decimal128 quantized128 (_Decimal128 __x, _Decimal128 __y) __THROW;
 extern _Decimal128 __quantized128 (_Decimal128 __x, _Decimal128 __y) __THROW;
 
+extern _Decimal128 quantumd128 (_Decimal128 __x) __THROW;
+extern _Decimal128 __quantumd128 (_Decimal128 __x) __THROW;
+
 extern _Bool samequantumd128 (_Decimal128 __x, _Decimal128 __y) __THROW;
 extern _Bool __samequantumd128 (_Decimal128 __x, _Decimal128 __y) __THROW;
 
@@ -963,6 +1011,35 @@ extern _Bool __samequantumd128 (_Decimal128 __x, _Decimal128 __y) __THROW;
 	: __isnand32(x)))						      \
   )
 
+
+/* Return nonzero value if X is a sNaN.  */
+#ifdef issignaling
+# undef issignaling
+# ifdef __NO_LONG_DOUBLE_MATH
+#  define ____issignaling(x) 				\
+     (sizeof (x) == sizeof (float) ? __issignalingf (x) \
+	: __issignaling (x))
+# else
+#  define ____issignaling(x)				\
+     (sizeof (x) == sizeof (float)			\
+      ? __issignalingf (x)				\
+      : sizeof (x) == sizeof (double)			\
+      ? __issignaling (x) : __issignalingl (x))
+# endif
+#else
+# define ____issignaling(x) x
+#endif
+
+#define issignaling(x)					\
+  (!__dfp_compatible(x)					\
+    ? (____issignaling(x))				\
+    : (sizeof (x) == sizeof (_Decimal128)		\
+      ? __issignalingd128(x)				\
+      : (sizeof (x) == sizeof (_Decimal64)		\
+	? __issignalingd64(x)				\
+	: __issignalingd32(x)))				\
+  )
+
 __END_DECLS
 
 #endif /* __STDC_WANT_DEC_FP__  */
