@@ -1,7 +1,7 @@
 /* Returns non-zero if the _Decimal128 is non-infinite
 
    Copyright (C) 2006 IBM Corporation.
-   Copyright (C) 2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2007-2014 Free Software Foundation, Inc.
 
    This file is part of the Decimal Floating Point C Library.
 
@@ -23,7 +23,26 @@
 
    Please see libdfp/COPYING.txt for more information.  */
 
-#define _DECIMAL_SIZE 128
-#include <decimal128.h>
+#include <math.h>
+#include <math_private.h>
+#include <ieee754r_private.h>
 
-#include "isfinited32.c"
+int
+__isfinited128 (_Decimal128 x)
+{
+  uint64_t hx;
+  GET_DEC128_HIGH_WORD64 (hx, x);
+
+  if (((hx & DEC128_NAN_MASK64) == DEC128_NAN_MASK64) ||
+      ((hx & DEC128_INF_MASK64) == DEC128_INF_MASK64))
+    return 0;
+  return 1;
+}
+hidden_def (__isfinited128)
+weak_alias (__isfinited128, isfinited128)
+
+/* We erroneously published a version of math.h which used 'finite' instead of
+ * 'isfinite' and math.h contained a polymorphic 'isfinite()' function which
+ * inlined calles to 'finited*' so we've created aliases for compatability.  */
+strong_alias (__isfinited128, finited128)
+strong_alias (__isfinited128, __finited128)
