@@ -1,6 +1,6 @@
 /* Function definition to convert DFP values to strings
 
-   Copyright (C) 2006, 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2006-2012 Free Software Foundation, Inc.
 
    This file is part of the Decimal Floating Point C Library.
 
@@ -219,6 +219,13 @@ padn (FILE *fp, int pad, int count)
       OUT("prec",pr); \
     } while (0)
 
+#define OUT_INPUT_PREC(pr) \
+  do \
+    { \
+      OUT("input_prec",pr); \
+    } while (0)
+
+
 #define OUT_INDEX(idx) \
   do \
     { \
@@ -319,7 +326,7 @@ __dfp_ais (const struct printf_info *info, size_t n __attribute__ ((unused)), in
       return 1;
     }
 
-  return 0;
+  return -1;
 }
 strong_alias(__dfp_ais, dfp_ais)
 hidden_def(__dfp_ais)
@@ -340,15 +347,11 @@ __printf_dfp (FILE *fp,
 	      const struct printf_info *info,
 	      const void *const *args)
 {
-	int wide = info->wide;
-	/* Counter for number of written characters.	*/
-	int done = 0;
+  int wide = info->wide;
+  /* Counter for number of written characters.  */
+  int done = 0;
 
-  /* Locale-dependent representation of decimal point.	*/
-
-#ifndef OPTION_EGLIBC_LOCALE_CODE
-  char decimald;
-#endif
+  /* Locale-dependent representation of decimal point.  */
   const char *decimal;
 
   union { const char *mb; unsigned int wc; } decimalwc;
@@ -383,12 +386,10 @@ __printf_dfp (FILE *fp,
   assert (decimalwc.wc != L'\0');
 #else
   /* Hard-code values from 'C' locale.  */
-  decimald = ".";
-  decimal = &decimald;
+  decimal = ".";
   decimalwc.wc = L'.';
 #endif
 
-
 #ifdef OPTION_EGLIBC_LOCALE_CODE
   if (info->group)
     {
@@ -476,7 +477,7 @@ __printf_dfp (FILE *fp,
       __get_digits_d128 (d128, digits+1, &exp, &is_neg, &is_nan, &is_inf);
       mw = __DEC128_MANT_DIG__ + 1;
     }
-  else
+  else if (info->user & mod_H)
     {
        _Decimal32 d32 = **(_Decimal32**)args[0];
        if (d32 == 0) is_zero = 1;
@@ -484,6 +485,8 @@ __printf_dfp (FILE *fp,
        __get_digits_d32 (d32, digits+1, &exp, &is_neg, &is_nan, &is_inf);
        mw = __DEC32_MANT_DIG__ + 1;
     }
+  else /* We shouldn't get here, but it is possible.  */
+    return -2;
 
   /* The first digit is always a zero to allow rounding.  */
   n = 0;
@@ -542,16 +545,9 @@ __printf_dfp (FILE *fp,
     {
       case 'a':
 	  {
-	    /* This is totally bogus but the DFP spec addition for %a refers to
-	     * all of the significant digits in the precision.  */
-	    if (exp < 0)
-	      {
-		  input_prec = nd;
-	      }
-	    else
-	      {
-	        input_prec = 1;
-	      }
+	    /* The DFP spec addition for %a refers to all of the significant
+	     * digits in the precision.  */
+	    input_prec = nd;
 
 	    /* This same check is done in two different places but it'll only
 	     * effect a single pass through once.  If prec is not set it'll hit
@@ -611,8 +607,8 @@ __printf_dfp (FILE *fp,
 	      {
 		prec = P - 1;
 		spec = 'e';
-		input_prec = nd - 1;
 	      }
+	    input_prec = nd - 1;
 	  break;
 	  }
 	case 'e':
@@ -650,8 +646,10 @@ __printf_dfp (FILE *fp,
 	index = n + nd + exp + prec;
       /* Goofy special case where we round significant digits which aren't
        * right of the decimal place.  */
-      else if (tolower(info->spec) == 'a')
+      else if (tolower(info->spec) == 'a' && prec > 0)
+       {
 	index = n + prec;
+       }
       else
 	index = n + prec + 1;
 
@@ -911,3 +909,4 @@ int __register_printf_dfp (void)
   return 0;
 }
 strong_alias (__register_printf_dfp, register_printf_dfp)
+hidden_def (__register_printf_dfp)
