@@ -1,5 +1,6 @@
 /* Returns non-zero if the _Decimal128 is infinite
    Copyright (C) 2008 IBM Corporation.
+   Copyright (C) 2014 Free Software Foundation, Inc.
 
    Author(s): Pete Eberlein <eberlein@us.ibm.com>
 
@@ -19,8 +20,25 @@
 
    Please see libdfp/COPYING.txt for more information.  */
 
-#define _DECIMAL_SIZE 128
-#define FUNCTION_NAME isinf
-#define TEST_CLASS_MASK 0x04
+#include <math.h>
+#include <ieee754r_private.h>
 
-#include "is_template.h"
+int
+__isinfd128 (_Decimal128 x)
+{
+  register _Decimal128 input asm("fr0") = x;
+  int cr0;
+
+  asm ("dtstdcq cr0,%1,0x04\n"
+       "mfcr     %0, 0\n"
+       : "=r" (cr0)
+       : "f" (input)
+       : "cr0");
+
+  /* cr0 bits are 28:31 and:
+     - 0010 operand positive with match
+     - 1010 operand negative with math  */
+  return (cr0 & 0x20000000) ? (cr0 & 0x80000000 ? -1 : 1) : 0;
+}
+hidden_def (__isinfd128)
+weak_alias (__isinfd128, isinfd128)
