@@ -1,6 +1,6 @@
 /* _Decimal64 isinf classification function.
 
-   Copyright (C) 2010 Free Software Foundation, Inc.
+   Copyright (C) 2010-2014 Free Software Foundation, Inc.
 
    This file is part of the Decimal Floating Point C Library.
 
@@ -23,5 +23,24 @@
 
    Please see libdfp/COPYING.txt for more information.  */
 
-#define _DECIMAL_SIZE 64
-#include "isinfd32.c"
+#include <math.h>
+#include <ieee754r_private.h>
+
+int
+__isinfd64 (_Decimal64 x)
+{
+  int cr0;
+
+  asm ("dtstdc cr0,%1,0x04\n"
+       "mfcr    %0, 0\n"
+       : "=r" (cr0)
+       : "f" (x)
+       : "cr0");
+
+  /* cr0 bits are 28:31 and:
+     - 0010 operand positive with match
+     - 1010 operand negative with math  */
+  return (cr0 & 0x20000000) ? (cr0 & 0x80000000 ? -1 : 1) : 0;
+}
+hidden_def (__isinfd64)
+weak_alias (__isinfd64, isinfd64)
