@@ -1,5 +1,7 @@
 /* Returns non-zero if the _Decimal128 is NaN
+ 
    Copyright (C) 2008 IBM Corporation.
+   Copyright (C) 2014 Free Software Foundation, Inc.
 
    Author(s): Pete Eberlein <eberlein@us.ibm.com>
 
@@ -19,8 +21,22 @@
 
    Please see libdfp/COPYING.txt for more information.  */
 
-#define _DECIMAL_SIZE 128
-#define FUNCTION_NAME isnan
-#define TEST_CLASS_MASK 0x03
+#include <math.h>
+#include <ieee754r_private.h>
 
-#include "is_template.h"
+int
+__isnand128 (_Decimal128 val)
+{
+  register _Decimal128 input asm("fr0") = val;
+  int cr0;
+
+  asm ("dtstdcq cr0,%1,3\n"
+       "mfcr     %0, 0\n"
+    : "=r" (cr0)
+    : "f" (input)
+    : "cr0");
+
+  return (cr0 & 0x20000000) ? 1 : 0;
+}
+hidden_def (__isnand128)
+weak_alias (__isnand128, isnand128)
