@@ -1,6 +1,6 @@
 /* Test printf_dfp facility.
 
-   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2009-2012 Free Software Foundation, Inc.
 
    This file is part of the Decimal Floating Point C Library.
 
@@ -116,6 +116,11 @@ d32_type printf_d32s[] =
   {__LINE__, 123.456E-9DF, "1.23456e-7", "%Ha"},
   {__LINE__, 0.00000E-101DF, "0.000000e+00", "%He"},
 
+  /* Erroneously prints "1e+6" due to the '.' with no explicit precision. FIX
+   * ME.  */
+  {__LINE__, 6.0E5DF, "6e+5", "%.Ha"},
+  {__LINE__, 6.0E5DF, "6e+5", "%.0Ha"},
+  {__LINE__, 6.E5DF, "6e+5", "%.0Ha"},
 
   {__LINE__, 0.0e10DF, "0.000000e+00", "%He"},
   {__LINE__, 0.0e10DF, "0.000000", "%Hf"},
@@ -214,6 +219,7 @@ d32_type printf_d32s[] =
   {__LINE__, 6543.00DF, "6543.00", "%Ha"},
   {__LINE__, 6543.00DF, "6543.00", "%.6Ha"},
   {__LINE__, 6543.00DF, "6543.0", "%.5Ha"},
+  {__LINE__, 6543.09DF, "6543.1", "%.5Ha"},
   {__LINE__, 6543.00DF, "6543", "%.4Ha"},
   {__LINE__, 6543.00DF, "6.54e+3", "%.3Ha"},
   {__LINE__, 6543.00DF, "6.5e+3", "%.2Ha"},
@@ -337,6 +343,10 @@ d64_type printf_d64s[] =
   {__LINE__, 6.5DD, "6e+00", "%.0De"},
   {__LINE__, 6.6DD, "7e+00", "%.0De"},
 
+  /* Erroneously prints "1e+6" due to the '.' with no explicit precision. Fix
+   * ME.  */
+  {__LINE__, 6.0E5DD, "6e+5", "%.Da"},
+
   {__LINE__, 123.456E-9DD, "1.23456e-7", "%Da"},
   {__LINE__, 0.00000E-101DD, "0.000000e+00", "%De"},
 
@@ -480,6 +490,24 @@ d64_type printf_d64s[] =
 
   {__LINE__, 6543.00DD, " 6543.00", "%8Da"},
 
+  {__LINE__, __DEC64_MAX__, "9.999999999999999e+384", "%.17Dg" },
+  {__LINE__, __DEC64_MAX__, "9.999999999999999e+384", "%Da" },
+  {__LINE__, __DEC64_MAX__, "1.000000e+385", "%De" },
+  {__LINE__, __DEC64_MAX__, "9.999999999999999e+384", "%.16Da" },
+  {__LINE__, __DEC64_MAX__, "1.00e+385", "%.3Da" },
+  {__LINE__, 9.999999999999999E16DD, "9.999999999999999e+16", "%.16Dg" },
+  {__LINE__, 9.999999999999999E23DD, "9.999999999999999e+23", "%.16Dg" },
+  {__LINE__, 9.999999999999999E300DD, "9.999999999999999e+300", "%.16Dg" },
+  {__LINE__, 9.999999999999999E360DD, "9.999999999999999e+360", "%.16Dg" },
+  {__LINE__, 9.999999999999999E369DD, "9.999999999999999e+369", "%.16Dg" },
+  {__LINE__, 9.999999999999999E370DD, "9.999999999999999e+370", "%.16Dg" },
+  {__LINE__, 9.999999999999999E379DD, "9.999999999999999e+379", "%.16Dg" },
+  {__LINE__, 9.999999999999999E380DD, "9.999999999999999e+380", "%.16Dg" },
+  {__LINE__, 9999999999999999E369DD, "9.999999999999999e+384", "%.16Dg" },
+  {__LINE__, 4E384DD, "4e+384", "%Dg" },
+  {__LINE__, 4E384DD, "4e+384", "%.Dg" },
+
+
   {0,0,0,0 }
 };
 
@@ -558,6 +586,10 @@ d128_type printf_d128s[] =
   {__LINE__, 6.5DL, "6e+00", "%.0DDe"},
   {__LINE__, 6.6DL, "7e+00", "%.0DDe"},
 
+  /* Erroneously prints "1e+6" due to the '.' with no explicit precision. FIX
+   * ME.  */
+  {__LINE__, 6.0E5DL, "6e+5", "%.DDa"},
+
   {__LINE__, 123.456E-9DL, "1.23456e-7", "%DDa"},
   {__LINE__, 0.00000E-101DL, "0.000000e+00", "%DDe"},
 
@@ -699,7 +731,7 @@ d128_type printf_d128s[] =
   {__LINE__, 6543.00DL, "7e+3", "%.1DDa"},
   {__LINE__, 6543.00DL, "7E+3", "%.1DDA"},
 
-  {__LINE__, 6543.00DL, " 6543.00", "%8DDa"},
+  {__LINE__, 6543.00DL, "6543.00", "%.8DDa"},
 
   /* Fixed by Andreas Schwab on 2010-20-04 */
   {__LINE__, 0.9999999999DL, "1.000000", "%DDf"},
@@ -708,6 +740,17 @@ d128_type printf_d128s[] =
    * It was fixed by Andreas Krebbel on 2011-01-18 */
   {__LINE__, (1.0DL / 1.000000e-123DL), "1000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000.000000", "%DDf"},
 
+  {__LINE__, -1234.56789123456789123455678DL, "-1234.56789123", "%.8DDf"},
+  /* This prints like f/F but g/G computes precision based on all digits in
+   * the mantissa, not just those to the right of the decimal place like
+   * regular f/F.  */
+  {__LINE__, -1234.56789123456789123455678DL, "-1234.5679", "%.8DDg"},
+  {__LINE__, -1234.56789123456789123455678DL, "-1234.56789123", "%.12DDg"},
+  /* This isn't properly rounding after the truncated.  FIX ME. */
+  {__LINE__, -1234.56789123456789123455678DL, "-1234.57", "%DDg"},
+  {__LINE__, -1234.56789123456789123455678DL, "-1234.5679", "%.8DDa"},
+  {__LINE__, -1234.56789123456789123455678DL, "-1.234568e+03", "%.6DDe"},
+  {__LINE__, -1234.56789123456789123455678DL, "-1.23456789e+03", "%.8DDe"},
   {0,0,0,0 }
 };
 
